#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <time.h>
#include <errno.h>
#include "basic.h"

/* Memory allocation */
void *malloc_check(size_t sz) {
  if (sz == 0) return NULL ;
  void *p = malloc(sz) ;
  if (!p) {
    perror("malloc") ;
    exit(2) ;
  }
  return p ;
}

void *calloc_check(size_t nmemb,size_t sz) {
  if (sz == 0) return NULL ;
  void *p = calloc(nmemb,sz) ;
  if (!p) {
    perror("calloc") ;
    exit(2) ;
  }
  return p ;
}


/* Specific error function */

void errexit(char *msg,int err) {
  fprintf(stderr,"%s: %s\n",msg,strerror(err)) ;
  exit(2) ;
}

/* Boxed int */

void free_boxed_int(boxed_int_t *p) { free(p) ; }

boxed_int_t *alloc_boxed_int(int v) {
  boxed_int_t *r = malloc_check(sizeof(*r)) ;
  r->v = v ;
  return r ;
}


/* Threads */

void create_thread(pthread_t *th,void *(*f)(void *),void *x) {
  int err ;
  if ((err = pthread_create(th,NULL,f,x))) {
    errexit("pthread_create",err) ;
  }
}

void create_thread_detached(pthread_t *th,void *(*f)(void *),void *x) {
  pthread_attr_t tattr;
  int err ;

  err = pthread_attr_init(&tattr) ;
  if (err) errexit("pthread_attr_init",err) ;

  err = pthread_attr_setdetachstate(&tattr,PTHREAD_CREATE_DETACHED) ;
  if (err) errexit("pthread_attr_setdetachstate",err) ;

  err = pthread_create(th,&tattr,f,x) ;
  if (err) errexit("pthread_create",err) ;

  err = pthread_attr_destroy(&tattr) ;
  if (err) errexit("pthread_attr_destroy",err) ;
}

void *join_thread(pthread_t *th) {
  void *r ;
  int err ;
  err = pthread_join (*th,&r) ;
  if (err) errexit("pthread_join",err) ;
  return r ;
}
/* Mutex */

pthread_mutex_t *alloc_mutex(void) {
  pthread_mutex_t *r = malloc_check(sizeof(*r)) ;
  int err = pthread_mutex_init(r,NULL) ;
  if (err) errexit(" pthread_mutex_init",err) ;
  return r ;
}

void free_mutex(pthread_mutex_t *p) {
  int err = pthread_mutex_destroy(p) ;
  if (err) errexit("pthread_mutex_destroy",err) ;
  free(p) ;
}

void lock_mutex(pthread_mutex_t *p) {
  int err = pthread_mutex_lock(p) ;
  if (err)  errexit("pthread_mutex_lock",err) ;
}

void unlock_mutex(pthread_mutex_t *p) {
  int err = pthread_mutex_unlock(p) ;
  if (err) errexit("pthread_mutex_unlock",err) ;
}

/* Condition variable */

pthread_cond_t *alloc_cond(void) {
  pthread_cond_t *r = malloc_check(sizeof(*r)) ;
  int err = pthread_cond_init(r,NULL) ;
  if (err) {
    errexit("pthread_cond_init",err) ;
   }
  return r ;
}

void free_cond(pthread_cond_t *p) {
  int err = pthread_cond_destroy(p) ;
  if (err) {
    errexit("pthread_cond_destroy",err) ;
  }
  free(p) ;
}

void wait_cond(pthread_cond_t *c, pthread_mutex_t *m) {
  int err = pthread_cond_wait(c,m) ;
  if (err) errexit("pthread_cond_wait",err) ;
}

void signal_cond(pthread_cond_t *c) {
  int err = pthread_cond_signal(c) ;
  if (err) errexit("pthread_cond_signal",err) ;
}

void broadcast_cond(pthread_cond_t *c) {
  int err = pthread_cond_broadcast(c) ;
  if (err) errexit("pthread_cond_broadcast",err);
}

/* Delay */

void delay(unsigned int ms) {
  struct timespec req,rem ;
  unsigned int sec = ms / 1000 ;
  long ns = (ms % 1000) * 100000L ;
  req.tv_sec = sec ; req.tv_nsec = ns ;
  while (nanosleep(&req,&rem)) {
    if (errno == EINTR) {
      req = rem ;
    } else {
      errexit("nanosleep",errno) ;
    }
  }
}

/* Timing info */

tsc_t timeofday(void) {
  struct timeval tv ;
  if (gettimeofday(&tv,NULL)) {
    errexit("gettimeofday",errno) ;
  }
  return tv.tv_sec * ((tsc_t)1000000) + tv.tv_usec ;
}

double tsc_millions(tsc_t t) {
  return t / 1000000.0 ;
}
